
function setup()
    
    viewer.mode = STANDARD
    craft.scene.main = craft.scene()
    scene = craft.scene.main
    scene.sky.material.sky = color(184, 213, 199)       
    scene.sky.material.horizon = color(22, 22, 35)       
    scene.sky.material.ground = color(52, 45, 100)
    
    scene.camera:add(OrbitViewer,vec3(0,0,0), 30, 0, 100)
    
    -- Create an instance of GridDrawer
    gridDrawer = GridDrawer(scene)
    
    randomlyPlaceSphereSomewhere()
end

function draw()    
    -- Draw the grid
    gridDrawer:draw()
end

function touched(touch)
    gridDrawer:touched(touch)
    if not gridDrawer.allowDrag then
        touches.touched(touch)
    end   
end

function randomlyPlaceSphereSomewhere()
    -- Create a sphere entity
    sphere = scene:entity()
    sphere.model = craft.model.icosphere(8.75, 2) -- Using a cube model for simplicity
    sphere.material = craft.material("Materials:Standard")
    sphere.material.map = readImage(asset.builtin.Surfaces.Stone_Brick_Roughness)
    sphere.scale = vec3(0.5, 0.5, 0.5) -- Set the size of the sphere
    
    -- Place the sphere at a random position within the grid
    local halfExtent = 24 * 0.5
    sphere.position = vec3(
    math.random(-halfExtent, halfExtent),
    math.random(-halfExtent, halfExtent),
    math.random(-halfExtent, halfExtent)
    )
end

-- Test function for HSVtoRGB
function test_HSVtoRGB()
    local testValues = {
        {h = 0, s = 1, v = 1},       -- Red
        {h = 1/3, s = 1, v = 1},   -- Green
        {h = 2/3, s = 1, v = 1},   -- Blue
        {h = 0.5, s = 0.5, v = 0.5} -- Some random color
    }
    
    for _, value in ipairs(testValues) do
        local r, g, b = HSVtoRGB(value.h, value.s, value.v)
        print("HSV -> RGB: ", value.h, value.s, value.v, " -> ", r, g, b)
    end
end

function testGetTriangleColors()
    local testColors = {
        {r = 255, g = 0, b = 0}, -- Red
        {r = 0, g = 255, b = 0}, -- Green
        {r = 0, g = 0, b = 255}, -- Blue
        {r = 127, g = 127, b = 127}, -- Gray
    }
    
    for _, chosenColor in ipairs(testColors) do
        print("Testing with color: ", chosenColor.r, chosenColor.g, chosenColor.b)
        local col1, col2, col3 = getTriangleColors(chosenColor)
        assert(col1.r <= 255 and col1.g <= 255 and col1.b <= 255, "Col1 values out of range!")
        assert(col2.r <= 255 and col2.g <= 255 and col2.b <= 255, "Col2 values out of range!")
        assert(col3.r <= 255 and col3.g <= 255 and col3.b <= 255, "Col3 values out of range!")
    end
    
    print("All tests passed!")
end



