-- Utility function to convert RGB to HSV
function RGBtoHSV(r, g, b)
    local max, min = math.max(r, g, b), math.min(r, g, b)
    local h, s, v = max, max, max - min
    if max == min then
        h = 0
    elseif max == r then
        h = (g - b) / (max - min)
    elseif max == g then
        h = 2 + (b - r) / (max - min)
    else
        h = 4 + (r - g) / (max - min)
    end
    h = h * 60
    if h < 0 then h = h + 360 end
    if max ~= 0 then
        s = (max - min) / max
    else
        s = 0
    end
    return h, s, v
end

function HSVtoRGB(h, s, v)
    local r, g, b
    local i = math.floor(h * 6)
    local f = h * 6 - i
    local p = v * (1 - s)
    local q = v * (1 - f * s)
    local t = v * (1 - (1 - f) * s)
    local switch = i % 6
    if switch == 0 then
        r = v
        g = t
        b = p
    elseif switch == 1 then
        r = q
        g = v
        b = p
    elseif switch == 2 then
        r = p
        g = v
        b = t
    elseif switch == 3 then
        r = p
        g = q
        b = v
    elseif switch == 4 then
        r = t
        g = p
        b = v
    elseif switch == 5 then
        r = v
        g = p
        b = q
    end
    return r*255, g*255, b*255  -- Multiply by 255 to get values in the range [0, 255]
end

function getTriangleColors(chosenColor)
    local hue, sat, bri = RGBtoHSV(chosenColor.r / 255, chosenColor.g / 255, chosenColor.b / 255) -- 
    
    -- Calculate the hue offset based on the current hue's distance from red
    local hueOffset = 120
    
    -- Calculate the other two hues using the hue offset
    local hue1 = (hue + hueOffset) % 360
    local hue2 = (hue + 2 * hueOffset) % 360
    
    local r1, g1, b1 = HSVtoRGB(hue / 360, sat, bri)
    local r2, g2, b2 = HSVtoRGB(hue1 / 360, sat, bri)
    local r3, g3, b3 = HSVtoRGB(hue2 / 360, sat, bri)
    
    -- Determine which color is closest to red (0° on the hue circle)
    local redHue = 0
    local hueDistances = {
        {dist = angularDistance(hue, redHue), index = 1},
        {dist = angularDistance(hue1, redHue), index = 2},
        {dist = angularDistance(hue2, redHue), index = 3}
    }
    table.sort(hueDistances, function(a, b) return a.dist < b.dist end)
    local closestToRedIndex = hueDistances[1].index
    
    -- Adjust brightness and saturation for the other two colors
    local adjustFactor = 0.48 -- Adjust this value to control the reduction amount
    if closestToRedIndex ~= 1 then
        r1, g1, b1 = HSVtoRGB(hue / 360, sat * adjustFactor, bri * adjustFactor)
    end
    if closestToRedIndex ~= 2 then
        r2, g2, b2 = HSVtoRGB(hue1 / 360, sat * adjustFactor, bri * adjustFactor)
    end
    if closestToRedIndex ~= 3 then
        r3, g3, b3 = HSVtoRGB(hue2 / 360, sat * adjustFactor, bri * adjustFactor)
    end
    
    local col1 = color(r1, g1, b1)
    local col2 = color(r2, g2, b2)
    local col3 = color(r3, g3, b3)
    
    --[[
    print("Col1: ", col1.r, col1.g, col1.b)
    print("Col2: ", col2.r, col2.g, col2.b)
    print("Col3: ", col3.r, col3.g, col3.b)
    ]]
    
    return col1, col2, col3
end

function angularDistance(hue1, hue2)
    local diff = math.abs(hue1 - hue2) % 360
    return math.min(diff, 360 - diff)
end
